@@ -335,6 +335,7 @@ describe("parser", function()
 	it("parses variable definitions", function()
 		assert.are.same({ { "UNKNOWN FILE", 1, "variable", "int", "var", nil, nil } }, parser.parse_text("int var;"));
 		assert.are.same({ { "UNKNOWN FILE", 1, "variable", "int", "array", nil, { "UNKNOWN FILE", 1, "num", 10 } } }, parser.parse_text("int array[10];"));
+		assert.are.same({ { "UNKNOWN FILE", 1, "variable", "int", "array", nil, { "UNKNOWN FILE", 1, "num", 10 } } }, parser.parse_text("int array[ 10 ];"));
 		
 		assert.are.same({ { "UNKNOWN FILE", 1, "variable", "struct foo", "bar", nil, nil } }, parser.parse_text("struct foo bar;"));
 		assert.are.same({ { "UNKNOWN FILE", 1, "variable", "struct baz", "qux", nil, { "UNKNOWN FILE", 1, "num", 10 } } }, parser.parse_text("struct baz qux[10];"));
@@ -1106,11 +1107,11 @@ describe("parser", function()
 		assert.are.same(expect, got)
 	end)
 	
-	it("errors on for loop with no non-local variable definition ini initialiser", function()
+	it("errors on for loop with no non-local variable definition in initialiser", function()
 		assert.has_error(
 			function()
-				parser.parse_text("for(int x;;)")
-			end, "Parse error at UNKNOWN FILE:1 (at 'for(int x;;')")
+				parser.parse_text("for(int x;;) {}")
+			end, "Cannot declare non-local variable in 'for' loop initialiser at UNKNOWN FILE:1 (at 'int x;;) {}')")
 	end)
 	
 	it("parses while loop", function()
