@@ -94,7 +94,9 @@ REHex::ByteRangeSet::const_iterator REHex::ByteRangeSet::find_first_in(off_t off
 		--i;
 	}
 	
-	off_t end = offset + length;
+	off_t end = (std::numeric_limits<off_t>::max() - length) < offset
+		? std::numeric_limits<off_t>::max()
+		: offset + length;
 	
 	for(; i != ranges.end() && (i->offset < end || end < offset); ++i)
 	{
@@ -177,6 +179,8 @@ bool REHex::ByteRangeSet::empty() const
 
 void REHex::ByteRangeSet::data_inserted(off_t offset, off_t length)
 {
+	REHEX_BYTERANGESET_CHECK_PRE(ranges.begin(), ranges.end());
+	
 	std::mutex lock;
 	std::vector<Range> insert_elem;
 	size_t insert_idx;
@@ -256,13 +260,17 @@ void REHex::ByteRangeSet::data_inserted(off_t offset, off_t length)
 		assert(insert_elem.size() == 1);
 		ranges.insert(std::next(ranges.begin(), insert_idx), insert_elem[0]);
 	}
+	
+	REHEX_BYTERANGESET_CHECK_POST(ranges.begin(), ranges.end());
 }
 
 void REHex::ByteRangeSet::data_erased(off_t offset, off_t length)
 {
+	REHEX_BYTERANGESET_CHECK_PRE(ranges.begin(), ranges.end());
+	
 	/* Find the range of elements overlapping the range to be erased. */
 	
-	auto next = std::lower_bound(ranges.begin(), ranges.end(), Range((offset + length), 0));
+	auto next = std::lower_bound(ranges.begin(), ranges.end(), Range((offset + length + 1), 0));
 	
 	std::vector<Range>::iterator erase_begin = next;
 	std::vector<Range>::iterator erase_end   = next;
@@ -271,7 +279,7 @@ void REHex::ByteRangeSet::data_erased(off_t offset, off_t length)
 	{
 		auto sb_prev = std::prev(erase_begin);
 		
-		if((sb_prev->offset + sb_prev->length) > offset)
+		if((sb_prev->offset + sb_prev->length) >= offset)
 		{
 			erase_begin = sb_prev;
 		}
@@ -314,6 +322,8 @@ void REHex::ByteRangeSet::data_erased(off_t offset, off_t length)
 		erase_end->offset -= length;
 		++erase_end;
 	}
+	
+	REHEX_BYTERANGESET_CHECK_POST(ranges.begin(), ranges.end());
 }
 
 REHex::ByteRangeSet REHex::ByteRangeSet::intersection(const ByteRangeSet &a, const ByteRangeSet &b)
@@ -365,6 +375,8 @@ REHex::ByteRangeSet REHex::ByteRangeSet::intersection(const ByteRangeSet &a, con
 		}
 	}
 	
+	REHEX_BYTERANGESET_CHECK(intersection.begin(), intersection.end());
+	
 	return intersection;
 }
 
