@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2019 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2019-2022 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -32,9 +32,10 @@ namespace REHex {
 	class CommentTreeModel: public wxDataViewModel
 	{
 		public:
-			CommentTreeModel(REHex::Document *document);
+			CommentTreeModel(SharedDocumentPointer &document, DocumentCtrl *document_ctrl);
 			
 			void refresh_comments();
+			int get_max_comment_depth() const;
 			static const NestedOffsetLengthMapKey *dv_item_to_key(const wxDataViewItem &item);
 			
 			virtual int Compare(const wxDataViewItem &item1, const wxDataViewItem &item2, unsigned int column, bool ascending) const override;
@@ -45,9 +46,11 @@ namespace REHex {
 			virtual void GetValue(wxVariant &variant, const wxDataViewItem &item, unsigned int col) const override;
 			virtual bool IsContainer(const wxDataViewItem &item) const override;
 			virtual bool SetValue(const wxVariant &variant, const wxDataViewItem &item, unsigned int col) override;
+			virtual bool HasContainerColumns(const wxDataViewItem &item) const override;
 			
 		private:
-			REHex::Document *document;
+			SharedDocumentPointer document;
+			SafeWindowPointer<DocumentCtrl> document_ctrl;
 			
 			struct CommentData;
 			typedef std::pair<const NestedOffsetLengthMapKey, CommentData> values_elem_t;
@@ -64,16 +67,20 @@ namespace REHex {
 			{
 				values_elem_t *parent;
 				std::set<values_elem_t*, ChildElemCompare> children;
+				bool is_container;
 				
 				std::shared_ptr<const wxString> text;
 				
-				CommentData(values_elem_t *parent, const std::shared_ptr<const wxString> &text): parent(parent), text(text) {}
+				CommentData(values_elem_t *parent, const std::shared_ptr<const wxString> &text): parent(parent), is_container(false), text(text) {}
 			};
 			
 			std::map<NestedOffsetLengthMapKey, CommentData> values;
 			std::set<values_elem_t*, ChildElemCompare> root;
 			
+			int max_comment_depth;
+			
 			std::map<NestedOffsetLengthMapKey, CommentData>::iterator erase_value(std::map<NestedOffsetLengthMapKey, CommentData>::iterator value_i);
+			void re_add_item(values_elem_t *value, bool as_container);
 	};
 	
 	class CommentTree: public ToolPanel
@@ -97,9 +104,11 @@ namespace REHex {
 			SafeWindowPointer<DocumentCtrl> document_ctrl;
 			
 			wxDataViewCtrl *dvc;
-			wxDataViewColumn *dvc_col;
+			wxDataViewColumn *offset_col, *text_col;
 			CommentTreeModel *model;
 			
+			int historic_max_comment_depth;
+			
 			void refresh_comments();
 			
 			void OnCommentModified(wxCommandEvent &event);
