@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2020-2021 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2020-2022 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -31,6 +31,7 @@
 #include "DiffWindow.hpp"
 #include "mainwindow.hpp"
 #include "Palette.hpp"
+#include "SafeWindowPointer.hpp"
 #include "util.hpp"
 
 #include "../res/icon16.h"
@@ -74,12 +75,15 @@ BEGIN_EVENT_TABLE(REHex::DiffWindow, wxFrame)
 	EVT_TIMER(ID_UPDATE_REGIONS_TIMER, REHex::DiffWindow::OnUpdateRegionsTimer)
 END_EVENT_TABLE()
 
+REHex::DiffWindow *REHex::DiffWindow::instance = NULL;
+
 REHex::DiffWindow::DiffWindow(wxWindow *parent):
 	wxFrame(parent, wxID_ANY, "Compare data - Reverse Engineers' Hex Editor", wxDefaultPosition, wxSize(740, 540)),
 	statbar(NULL),
 	sb_gauge(NULL),
 	enable_folding(true),
 	recalc_bytes_per_line_pending(false),
+	update_regions_timer(this, ID_UPDATE_REGIONS_TIMER),
 	relative_cursor_pos(0),
 	longest_range(0),
 	searching_backwards(false),
@@ -148,8 +152,6 @@ REHex::DiffWindow::DiffWindow(wxWindow *parent):
 	
 	statbar = CreateStatusBar(2);
 	sb_gauge = new wxGauge(statbar, wxID_ANY, 100);
-	
-	update_regions_timer = new wxTimer(this, ID_UPDATE_REGIONS_TIMER);
 }
 
 REHex::DiffWindow::~DiffWindow()
@@ -176,6 +178,11 @@ REHex::DiffWindow::~DiffWindow()
 		
 		d->first->Unbind(DOCUMENT_TITLE_CHANGED, &REHex::DiffWindow::OnDocumentTitleChange, this);
 	}
+	
+	if(instance == this)
+	{
+		instance = NULL;
+	}
 }
 
 const std::list<REHex::DiffWindow::Range> &REHex::DiffWindow::get_ranges() const
@@ -360,7 +367,7 @@ std::list<REHex::DiffWindow::Range>::iterator REHex::DiffWindow::remove_range(st
 		*/
 
 		std::vector<DocumentCtrl::Region*> regions;
-		regions.push_back(new DocumentCtrl::DataRegion(0, 0, 0));
+		regions.push_back(new DocumentCtrl::DataRegion(range->doc, 0, 0, 0));
 
 		range->doc_ctrl->replace_all_regions(regions);
 	}
@@ -501,7 +508,7 @@ void REHex::DiffWindow::doc_update(Range *range)
 			 * to stop bad things from happening.
 			*/
 			
-			regions.push_back(new InvisibleDataRegion((range->offset + range->length), 0));
+			regions.push_back(new InvisibleDataRegion(range->doc, (range->offset + range->length), 0));
 		}
 	}
 	else{
@@ -681,9 +688,9 @@ off_t REHex::DiffWindow::process_now(off_t rel_offset, off_t length)
 	
 	offsets_pending.clear_range(rel_offset, length);
 	
-	if(!update_regions_timer->IsRunning())
+	if(!update_regions_timer.IsRunning())
 	{
-		update_regions_timer->StartOnce(100);
+		update_regions_timer.StartOnce(100);
 	}
 	
 	return length;
@@ -1399,7 +1406,19 @@ void REHex::DiffWindow::OnDataRightClick(wxCommandEvent &event)
 			source_range->doc->set_cursor_position(cursor_pos);
 			source_range->main_doc_ctrl->SetFocus();
 			
-			window->Raise();
+			/* Wait until the menu is gone before bringing the MainWindow to the top
+			 * or else we fight with it for focus.
+			*/
+			
+			SafeWindowPointer<MainWindow> w(window);
+			CallAfter([w]()
+			{
+				if(w)
+				{
+					w->Show();
+					w->Raise();
+				}
+			});
 		}
 		else{
 			/* Offset isn't currently available in main DocumentCtrl. */
@@ -1477,7 +1496,7 @@ void REHex::DiffWindow::OnUpdateRegionsTimer(wxTimerEvent &event)
 }
 
 REHex::DiffWindow::DiffDataRegion::DiffDataRegion(off_t d_offset, off_t d_length, DiffWindow *diff_window, Range *range):
-	DataRegion(d_offset, d_length, d_offset), diff_window(diff_window), range(range) {}
+	DataRegion(range->doc, d_offset, d_length, d_offset), diff_window(diff_window), range(range) {}
 
 int REHex::DiffWindow::DiffDataRegion::calc_width(REHex::DocumentCtrl &doc)
 {
@@ -1502,9 +1521,8 @@ REHex::DocumentCtrl::DataRegion::Highlight REHex::DiffWindow::DiffDataRegion::hi
 	if(diff_window->offsets_different.isset(relative_off))
 	{
 		return Highlight(
-			Palette::PAL_DIRTY_TEXT_FG,
-			Palette::PAL_DIRTY_TEXT_BG,
-			true);
+			(*active_palette)[Palette::PAL_DIRTY_TEXT_FG],
+			(*active_palette)[Palette::PAL_DIRTY_TEXT_BG]);
 	}
 	else{
 		return NoHighlight();
@@ -1568,7 +1586,7 @@ void REHex::DiffWindow::MessageRegion::draw(DocumentCtrl &doc_ctrl, wxDC &dc, in
 	dc.DrawText(message, x + text_x, y + (text_height / 2));
 }
 
-REHex::DiffWindow::InvisibleDataRegion::InvisibleDataRegion(off_t d_offset, off_t d_length):
-	DataRegion(d_offset, d_length, d_offset) {}
+REHex::DiffWindow::InvisibleDataRegion::InvisibleDataRegion(SharedDocumentPointer &document, off_t d_offset, off_t d_length):
+	DataRegion(document, d_offset, d_length, d_offset) {}
 
 void REHex::DiffWindow::InvisibleDataRegion::draw(REHex::DocumentCtrl &doc_ctrl, wxDC &dc, int x, int64_t y) {}
