@@ -90,6 +90,11 @@ wxColour REHex::Palette::get_average_colour(int colour_a_idx, int colour_b_idx)
 	const wxColour &colour_a = (*this)[colour_a_idx];
 	const wxColour &colour_b = (*this)[colour_b_idx];
 	
+	return get_average_colour(colour_a, colour_b);
+}
+
+wxColour REHex::Palette::get_average_colour(const wxColour &colour_a, const wxColour &colour_b)
+{
 	return wxColour(
 		(((int)(colour_a.Red())   + (int)(colour_b.Red()))   / 2),
 		(((int)(colour_a.Green()) + (int)(colour_b.Green())) / 2),
