@@ -112,6 +112,11 @@ namespace REHex {
 			*/
 			wxColour get_average_colour(int colour_a_idx, int colour_b_idx) const;
 			
+			/**
+			 * @brief Blend two colours together.
+			*/
+			static wxColour get_average_colour(const wxColour &colour_a, const wxColour &colour_b);
+			
 			static Palette *create_system_palette();
 			static Palette *create_light_palette();
 			static Palette *create_dark_palette();
