@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2019 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2019-2022 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -32,6 +32,13 @@
 
 #include "../src/CommentTree.hpp"
 #include "../src/document.hpp"
+#include "../src/DocumentCtrl.hpp"
+#include "../src/SharedDocumentPointer.hpp"
+
+#define MODEL_OFFSET_COLUMN 0
+#define MODEL_TEXT_COLUMN 1
+
+using namespace REHex;
 
 static const REHex::CommentTreeModel *build_model_values_cmp_model;
 static int build_model_values_cmp(wxDataViewItem *a, wxDataViewItem *b)
@@ -57,10 +64,13 @@ static void build_model_values(std::vector<std::string> &values, const std::stri
 	{
 		wxDataViewItem item = items[i];
 		
-		wxVariant value;
-		model->GetValue(value, item, 0);
+		wxVariant offset_value;
+		model->GetValue(offset_value, item, MODEL_OFFSET_COLUMN);
 		
-		std::string item_string = prefix + value.GetString().ToStdString();
+		wxVariant text_value;
+		model->GetValue(text_value, item, MODEL_TEXT_COLUMN);
+		
+		std::string item_string = prefix + offset_value.GetString().ToStdString() + "+" + text_value.GetString().ToStdString();
 		
 		values.push_back(item_string);
 		
@@ -102,10 +112,10 @@ struct TestDataViewModelNotifier: public wxDataViewModelNotifier
 			return "(null)";
 		}
 		
-		wxVariant value;
-		model->GetValue(value, item, 0);
+		wxVariant text_value;
+		model->GetValue(text_value, item, MODEL_TEXT_COLUMN);
 		
-		return value.GetString().ToStdString();
+		return text_value.GetString().ToStdString();
 	}
 	
 	void populate_items(wxDataViewItem parent)
@@ -120,7 +130,7 @@ struct TestDataViewModelNotifier: public wxDataViewModelNotifier
 			assert(item.IsOk());
 			
 			wxVariant value;
-			model->GetValue(value, item, 0);
+			model->GetValue(value, item, MODEL_TEXT_COLUMN);
 			
 			assert(this->items.find(item.GetID()) == this->items.end());
 			this->items.emplace(item.GetID(), value.GetString().ToStdString());
@@ -150,8 +160,12 @@ struct TestDataViewModelNotifier: public wxDataViewModelNotifier
 		assert(item.IsOk());
 		assert(items.find(item.GetID()) == items.end());
 		
+		const char *iscontainer = model->IsContainer(item)
+			? " (container)"
+			: "";
+		
 		items.emplace(item.GetID(), item_string(item));
-		events.push_back(std::string("ItemAdded(\"") + item_string(parent) + "\", \"" + item_string(item) + "\")");
+		events.push_back(std::string("ItemAdded(\"") + item_string(parent) + "\", \"" + item_string(item) + "\"" + iscontainer + ")");
 		return true;
 	}
 	
@@ -234,14 +248,17 @@ static void refresh_check_notifications(REHex::CommentTreeModel *model, const st
 
 TEST(CommentTree, NoComments)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
+	doc->insert_data(0, z1k, 1024);
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	
 	refresh_check_notifications(model, [](){}, NULL);
 	
@@ -252,15 +269,18 @@ TEST(CommentTree, NoComments)
 
 TEST(CommentTree, SingleComment)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(0, 0, REHex::Document::Comment("test"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(0, 0, REHex::Document::Comment("test"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	
 	refresh_check_notifications(model, [](){},
 		"ItemAdded(\"(null)\", \"test\")",
@@ -268,7 +288,7 @@ TEST(CommentTree, SingleComment)
 	);
 	
 	check_values(model,
-		"test",
+		"0000:0000+test",
 		NULL
 	);
 	
@@ -277,17 +297,20 @@ TEST(CommentTree, SingleComment)
 
 TEST(CommentTree, MultipleComments)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(0, 0,   REHex::Document::Comment("foo"));
-	doc.set_comment(10, 10, REHex::Document::Comment("bar"));
-	doc.set_comment(20, 0,  REHex::Document::Comment("baz"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(0, 0,   REHex::Document::Comment("foo"));
+	doc->set_comment(10, 10, REHex::Document::Comment("bar"));
+	doc->set_comment(20, 0,  REHex::Document::Comment("baz"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	
 		refresh_check_notifications(model, [](){},
 		"ItemAdded(\"(null)\", \"foo\")",
@@ -297,9 +320,9 @@ TEST(CommentTree, MultipleComments)
 	);
 	
 	check_values(model,
-		"foo",
-		"bar",
-		"baz",
+		"0000:0000+foo",
+		"0000:000A+bar",
+		"0000:0014+baz",
 		NULL
 	);
 	
@@ -308,38 +331,49 @@ TEST(CommentTree, MultipleComments)
 
 TEST(CommentTree, Heirarchy)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(10, 10, REHex::Document::Comment("10,10"));
-	doc.set_comment(10, 4,  REHex::Document::Comment("10,4"));
-	doc.set_comment(10, 0,  REHex::Document::Comment("10,0"));
-	doc.set_comment(13, 0,  REHex::Document::Comment("13,0"));
-	doc.set_comment(16, 4,  REHex::Document::Comment("16,4"));
-	doc.set_comment(19, 0,  REHex::Document::Comment("19,0"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(10, 10, REHex::Document::Comment("10,10"));
+	doc->set_comment(10, 4,  REHex::Document::Comment("10,4"));
+	doc->set_comment(10, 0,  REHex::Document::Comment("10,0"));
+	doc->set_comment(13, 0,  REHex::Document::Comment("13,0"));
+	doc->set_comment(16, 4,  REHex::Document::Comment("16,4"));
+	doc->set_comment(19, 0,  REHex::Document::Comment("19,0"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	
 	refresh_check_notifications(model, [](){},
 		"ItemAdded(\"(null)\", \"10,10\")",
+		"ItemDeleted(\"(null)\", \"10,10\")",
+		"ItemAdded(\"(null)\", \"10,10\" (container))",
 		"ItemAdded(\"10,10\", \"10,4\")",
+		"ItemAdded(\"10,10\", \"\")",
+		"ItemDeleted(\"10,10\", \"10,4\")",
+		"ItemAdded(\"10,10\", \"10,4\" (container))",
+		"ItemDeleted(\"10,10\", \"\")",
 		"ItemAdded(\"10,4\", \"10,0\")",
 		"ItemAdded(\"10,4\", \"13,0\")",
 		"ItemAdded(\"10,10\", \"16,4\")",
+		"ItemDeleted(\"10,10\", \"16,4\")",
+		"ItemAdded(\"10,10\", \"16,4\" (container))",
 		"ItemAdded(\"16,4\", \"19,0\")",
 		NULL
 	);
 	
 	check_values(model,
-		"10,10",
-		"10,10/10,4",
-		"10,10/10,4/10,0",
-		"10,10/10,4/13,0",
-		"10,10/16,4",
-		"10,10/16,4/19,0",
+		"0000:000A+10,10",
+		"0000:000A+10,10/0000:000A+10,4",
+		"0000:000A+10,10/0000:000A+10,4/0000:000A+10,0",
+		"0000:000A+10,10/0000:000A+10,4/0000:000D+13,0",
+		"0000:000A+10,10/0000:0010+16,4",
+		"0000:000A+10,10/0000:0010+16,4/0000:0013+19,0",
 		NULL
 	);
 	
@@ -348,22 +382,25 @@ TEST(CommentTree, Heirarchy)
 
 TEST(CommentTree, EraseRootCommentNoChildren)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(10, 10, REHex::Document::Comment("10,10"));
-	doc.set_comment(20, 10, REHex::Document::Comment("20,10"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(10, 10, REHex::Document::Comment("10,10"));
+	doc->set_comment(20, 10, REHex::Document::Comment("20,10"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	model->refresh_comments();
 	
 	refresh_check_notifications(model,
 		[&]()
 		{
-			doc.erase_comment(10, 10);
+			doc->erase_comment(10, 10);
 		},
 		
 		"ItemDeleted(\"(null)\", \"10,10\")",
@@ -371,7 +408,7 @@ TEST(CommentTree, EraseRootCommentNoChildren)
 	);
 	
 	check_values(model,
-		"20,10",
+		"0000:0014+20,10",
 		NULL
 	);
 	
@@ -380,38 +417,59 @@ TEST(CommentTree, EraseRootCommentNoChildren)
 
 TEST(CommentTree, EraseRootCommentWithChildren)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(10, 10, REHex::Document::Comment("10,10"));
-	doc.set_comment(12, 6,  REHex::Document::Comment("12,6"));
-	doc.set_comment(14, 0,  REHex::Document::Comment("14,0"));
-	doc.set_comment(20, 10, REHex::Document::Comment("20,10"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(10, 10, REHex::Document::Comment("10,10"));
+	doc->set_comment(12, 6,  REHex::Document::Comment("12,6"));
+	doc->set_comment(14, 0,  REHex::Document::Comment("14,0"));
+	doc->set_comment(20, 10, REHex::Document::Comment("20,10"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	model->refresh_comments();
 	
 	refresh_check_notifications(model,
 		[&]()
 		{
-			doc.erase_comment(10, 10);
+			doc->erase_comment(10, 10);
 		},
 		
+		/* Deleting 14,0, 12,6 is no longer a container... */
 		"ItemDeleted(\"12,6\", \"14,0\")",
+		"ItemAdded(\"10,10\", \"\")",
+		"ItemDeleted(\"10,10\", \"12,6\")",
+		"ItemAdded(\"10,10\", \"12,6\")",
+		"ItemDeleted(\"10,10\", \"\")",
+		
+		/* Deleting 12,6, 10,10 is no longer a container... */
 		"ItemDeleted(\"10,10\", \"12,6\")",
 		"ItemDeleted(\"(null)\", \"10,10\")",
+		"ItemAdded(\"(null)\", \"10,10\")",
+		
+		/* Now delete 10,10 for real... */
+		"ItemDeleted(\"(null)\", \"10,10\")",
+		
+		/* Add 12,6 back... */
 		"ItemAdded(\"(null)\", \"12,6\")",
+		
+		/* Add 14,0 back, 12,6 becomes a container again... */
+		"ItemDeleted(\"(null)\", \"12,6\")",
+		"ItemAdded(\"(null)\", \"12,6\" (container))",
 		"ItemAdded(\"12,6\", \"14,0\")",
+		
 		NULL
 	);
 	
 	check_values(model,
-		"12,6",
-		"12,6/14,0",
-		"20,10",
+		"0000:000C+12,6",
+		"0000:000C+12,6/0000:000E+14,0",
+		"0000:0014+20,10",
 		NULL
 	);
 	
@@ -420,26 +478,29 @@ TEST(CommentTree, EraseRootCommentWithChildren)
 
 TEST(CommentTree, EraseNestedCommentNoChildren)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(10, 10, REHex::Document::Comment("10,10"));
-	doc.set_comment(12, 0,  REHex::Document::Comment("12,0"));
-	doc.set_comment(20, 10, REHex::Document::Comment("20,10"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(10, 10, REHex::Document::Comment("10,10"));
+	doc->set_comment(12, 0,  REHex::Document::Comment("12,0"));
+	doc->set_comment(20, 10, REHex::Document::Comment("20,10"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	model->refresh_comments();
 	
-	doc.erase_comment(12, 0);
+	doc->erase_comment(12, 0);
 	
 	model->refresh_comments();
 	
 	check_values(model,
-		"10,10",
-		"20,10",
+		"0000:000A+10,10",
+		"0000:0014+20,10",
 		NULL
 	);
 	
@@ -448,30 +509,33 @@ TEST(CommentTree, EraseNestedCommentNoChildren)
 
 TEST(CommentTree, EraseNestedCommentWithChildren)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(10, 10, REHex::Document::Comment("10,10"));
-	doc.set_comment(12, 8,  REHex::Document::Comment("12,8"));
-	doc.set_comment(14, 6,  REHex::Document::Comment("14,6"));
-	doc.set_comment(16, 0,  REHex::Document::Comment("16,0"));
-	doc.set_comment(20, 10, REHex::Document::Comment("20,10"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(10, 10, REHex::Document::Comment("10,10"));
+	doc->set_comment(12, 8,  REHex::Document::Comment("12,8"));
+	doc->set_comment(14, 6,  REHex::Document::Comment("14,6"));
+	doc->set_comment(16, 0,  REHex::Document::Comment("16,0"));
+	doc->set_comment(20, 10, REHex::Document::Comment("20,10"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	model->refresh_comments();
 	
-	doc.erase_comment(12, 8);
+	doc->erase_comment(12, 8);
 	
 	model->refresh_comments();
 	
 	check_values(model,
-		"10,10",
-		"10,10/14,6",
-		"10,10/14,6/16,0",
-		"20,10",
+		"0000:000A+10,10",
+		"0000:000A+10,10/0000:000E+14,6",
+		"0000:000A+10,10/0000:000E+14,6/0000:0010+16,0",
+		"0000:0014+20,10",
 		NULL
 	);
 	
@@ -480,22 +544,25 @@ TEST(CommentTree, EraseNestedCommentWithChildren)
 
 TEST(CommentTree, AddCommentRoot)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(10, 10, REHex::Document::Comment("10,10"));
-	doc.set_comment(20, 10, REHex::Document::Comment("20,10"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(10, 10, REHex::Document::Comment("10,10"));
+	doc->set_comment(20, 10, REHex::Document::Comment("20,10"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	model->refresh_comments();
 	
 	refresh_check_notifications(model,
 		[&]()
 		{
-			doc.set_comment(30, 10, REHex::Document::Comment("30,10"));
+			doc->set_comment(30, 10, REHex::Document::Comment("30,10"));
 		},
 		
 		"ItemAdded(\"(null)\", \"30,10\")",
@@ -503,9 +570,9 @@ TEST(CommentTree, AddCommentRoot)
 	);
 	
 	check_values(model,
-		"10,10",
-		"20,10",
-		"30,10",
+		"0000:000A+10,10",
+		"0000:0014+20,10",
+		"0000:001E+30,10",
 		NULL
 	);
 	
@@ -514,32 +581,37 @@ TEST(CommentTree, AddCommentRoot)
 
 TEST(CommentTree, AddNestedComment)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(10, 10, REHex::Document::Comment("10,10"));
-	doc.set_comment(20, 10, REHex::Document::Comment("20,10"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(10, 10, REHex::Document::Comment("10,10"));
+	doc->set_comment(20, 10, REHex::Document::Comment("20,10"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	model->refresh_comments();
 	
 	refresh_check_notifications(model,
 		[&]()
 		{
-			doc.set_comment(22, 6, REHex::Document::Comment("22,6"));
+			doc->set_comment(22, 6, REHex::Document::Comment("22,6"));
 		},
 		
+		"ItemDeleted(\"(null)\", \"20,10\")",
+		"ItemAdded(\"(null)\", \"20,10\" (container))",
 		"ItemAdded(\"20,10\", \"22,6\")",
 		NULL
 	);
 	
 	check_values(model,
-		"10,10",
-		"20,10",
-		"20,10/22,6",
+		"0000:000A+10,10",
+		"0000:0014+20,10",
+		"0000:0014+20,10/0000:0016+22,6",
 		NULL
 	);
 	
@@ -548,48 +620,87 @@ TEST(CommentTree, AddNestedComment)
 
 TEST(CommentTree, AddContainingComment)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(10, 10, REHex::Document::Comment("10,10"));
-	doc.set_comment(12,  2, REHex::Document::Comment("12,2"));
-	doc.set_comment(14,  6, REHex::Document::Comment("14,6"));
-	doc.set_comment(15,  2, REHex::Document::Comment("15,2"));
-	doc.set_comment(20, 10, REHex::Document::Comment("20,10"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(10, 10, REHex::Document::Comment("10,10"));
+	doc->set_comment(12,  2, REHex::Document::Comment("12,2"));
+	doc->set_comment(14,  6, REHex::Document::Comment("14,6"));
+	doc->set_comment(15,  2, REHex::Document::Comment("15,2"));
+	doc->set_comment(20, 10, REHex::Document::Comment("20,10"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	model->refresh_comments();
 	
 	refresh_check_notifications(model,
 		[&]()
 		{
-			doc.set_comment(5, 25, REHex::Document::Comment("5,25"));
+			doc->set_comment(5, 25, REHex::Document::Comment("5,25"));
 		},
 		
+		/* Add 5,25... */
 		"ItemAdded(\"(null)\", \"5,25\")",
+		
+		/* Remove 12,2... */
 		"ItemDeleted(\"10,10\", \"12,2\")",
+		
+		/* Remove 15,2, 14,6 no longer a container... */
 		"ItemDeleted(\"14,6\", \"15,2\")",
+		"ItemAdded(\"10,10\", \"\")",
+		"ItemDeleted(\"10,10\", \"14,6\")",
+		"ItemAdded(\"10,10\", \"14,6\")",
+		"ItemDeleted(\"10,10\", \"\")",
+		
+		/* Remove 14,6, 10,10 no longer a container... */
 		"ItemDeleted(\"10,10\", \"14,6\")",
 		"ItemDeleted(\"(null)\", \"10,10\")",
+		"ItemAdded(\"(null)\", \"10,10\")",
+		
+		/* Remove 10,10... */
+		"ItemDeleted(\"(null)\", \"10,10\")",
+		
+		/* Add 10,10 back, 5,25 becomes a container... */
+		"ItemDeleted(\"(null)\", \"5,25\")",
+		"ItemAdded(\"(null)\", \"5,25\" (container))",
 		"ItemAdded(\"5,25\", \"10,10\")",
+		
+		/* Add 12,2 back, 10,10 becomes a container... */
+		"ItemAdded(\"5,25\", \"\")",
+		"ItemDeleted(\"5,25\", \"10,10\")",
+		"ItemAdded(\"5,25\", \"10,10\" (container))",
+		"ItemDeleted(\"5,25\", \"\")",
 		"ItemAdded(\"10,10\", \"12,2\")",
+		
+		/* Add 14,6... */
 		"ItemAdded(\"10,10\", \"14,6\")",
+		
+		/* Add 15,2, 14,6 becomes a container... */
+		"ItemDeleted(\"10,10\", \"14,6\")",
+		"ItemAdded(\"10,10\", \"14,6\" (container))",
 		"ItemAdded(\"14,6\", \"15,2\")",
+		
+		/* Remove 20,10... */
 		"ItemDeleted(\"(null)\", \"20,10\")",
+		
+		/* Add 20,10 back... */
 		"ItemAdded(\"5,25\", \"20,10\")",
+		
 		NULL
 	);
 	
 	check_values(model,
-		"5,25",
-		"5,25/10,10",
-		"5,25/10,10/12,2",
-		"5,25/10,10/14,6",
-		"5,25/10,10/14,6/15,2",
-		"5,25/20,10",
+		"0000:0005+5,25",
+		"0000:0005+5,25/0000:000A+10,10",
+		"0000:0005+5,25/0000:000A+10,10/0000:000C+12,2",
+		"0000:0005+5,25/0000:000A+10,10/0000:000E+14,6",
+		"0000:0005+5,25/0000:000A+10,10/0000:000E+14,6/0000:000F+15,2",
+		"0000:0005+5,25/0000:0014+20,10",
 		NULL
 	);
 	
@@ -598,22 +709,25 @@ TEST(CommentTree, AddContainingComment)
 
 TEST(CommentTree, ModifyComment)
 {
-	REHex::Document doc;
+	SharedDocumentPointer doc(SharedDocumentPointer::make());
+	
+	wxFrame frame(NULL, wxID_ANY, "REHex Tests");
+	DocumentCtrl *doc_ctrl = new DocumentCtrl(&frame, doc);
 	
 	unsigned char z1k[1024];
 	memset(z1k, 0, 1024);
 	
-	doc.insert_data(0, z1k, 1024);
-	doc.set_comment(10, 10, REHex::Document::Comment("10,10"));
-	doc.set_comment(20, 10, REHex::Document::Comment("20,10"));
+	doc->insert_data(0, z1k, 1024);
+	doc->set_comment(10, 10, REHex::Document::Comment("10,10"));
+	doc->set_comment(20, 10, REHex::Document::Comment("20,10"));
 	
-	REHex::CommentTreeModel *model = new REHex::CommentTreeModel(&doc);
+	CommentTreeModel *model = new CommentTreeModel(doc, doc_ctrl);
 	model->refresh_comments();
 	
 	refresh_check_notifications(model,
 		[&]()
 		{
-			doc.set_comment(10, 10, REHex::Document::Comment("hello"));
+			doc->set_comment(10, 10, REHex::Document::Comment("hello"));
 		},
 		
 		"ItemChanged(\"10,10\")",
@@ -621,8 +735,8 @@ TEST(CommentTree, ModifyComment)
 	);
 	
 	check_values(model,
-		"hello",
-		"20,10",
+		"0000:000A+hello",
+		"0000:0014+20,10",
 		NULL
 	);
 	
